#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Common implementation for all hooks.
"""

import jujuresources


def bootstrap_resources():
    """
    Attempt to load and install resources defined in resources.yaml
    """
    mirror_url = jujuresources.config_get('resources_mirror')
    if not jujuresources.fetch(mirror_url=mirror_url):
        jujuresources.juju_log('Required resources unavailable; changing resources_mirror option is required', 'ERROR')
        return False
    jujuresources.install(['pathlib', 'pyaml', 'six', 'charmhelpers', 'jujubigdata'])
    return True


def manage():
    if not bootstrap_resources():
        # defer until resources are available, since charmhelpers, and thus
        # the framework, are required (will require manual intervention)
        return

    from charmhelpers.core import charmframework
    import jujubigdata
    import callbacks  # noqa  (ignore when linting)

    # list of keys required to be in the dist.yaml
    yarn_reqs = ['vendor', 'hadoop_version', 'packages', 'groups',
                 'users', 'dirs', 'ports']
    dist_config = jujubigdata.utils.DistConfig(filename='dist.yaml',
                                               required_keys=yarn_reqs)
    hadoop = jujubigdata.handlers.HadoopBase(dist_config)
    yarn = jujubigdata.handlers.YARN(hadoop)
    hdfs = jujubigdata.handlers.HDFS(hadoop)
    port = dist_config.port('resourcemanager')
    nodemanagers = jujubigdata.relations.ResourceManagerMaster(spec=hadoop.spec, port=port)
    clients = jujubigdata.relations.ResourceManager(spec=hadoop.spec, port=port)
    manager = charmframework.Manager([
        {
            'name': 'hadoop-base',
            'requires': [
                hadoop.verify_conditional_resources,
            ],
            'callbacks': [
                hadoop.install,
            ],
        },
        {
            'name': 'yarn-master',
            'provides': [
                nodemanagers,
                clients,
            ],
            'requires': [
                hadoop.is_installed,
                jujubigdata.relations.NameNode(spec=hadoop.client_spec),
                jujubigdata.relations.NodeManager(optional=True),
            ],
            'callbacks': [
                nodemanagers.register_host_ips,
                jujubigdata.utils.update_etc_hosts_from_kv,
                hdfs.configure_client,
                yarn.configure_resourcemanager,
                yarn.configure_jobhistory,
                yarn.register_slaves,
                yarn.start_resourcemanager,
                yarn.start_jobhistory,
                charmframework.helpers.open_ports(dist_config.exposed_ports('yarn-master')),
            ],
            'cleanup': [
                charmframework.helpers.close_ports(dist_config.exposed_ports('yarn-master')),
                yarn.stop_resourcemanager,
                yarn.stop_jobhistory,
            ],
        },
    ])
    manager.manage()


if __name__ == '__main__':
    manage()
